/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#ifndef INTERFACEGUI_H
#define INTERFACEGUI_H

//-----------------------------------------------------------------------------

#include <QString>
#include <QPolygon>
#include <QPoint>
#include <QList>

//-----------------------------------------------------------------------------

class InterfaceGUI
{
public:

  virtual void TableClear() = 0;
  virtual void TableSetSize(int rows, int columns) = 0;
  virtual void TableSetValue(int row, int column, const QString & value) = 0;
  virtual void TableSetValuesInRow(int row, const QList <QString> & valueList) = 0;
  virtual void TableSetValuesInColumn(int column, const QList <QString> & valueList) = 0;
  virtual void TableSaveCsv(const QString & fileName) = 0;

  virtual void GfxClear() = 0;
  virtual void GfxSetSize(int x, int y) = 0;
  virtual void GfxDrawPoint(int x, int y) = 0;
  virtual void GfxDrawLine(int x1, int y1, int x2, int y2) = 0;
  virtual void GfxDrawCircle(int x, int y, int r) = 0;
  virtual void GfxDrawEllipse(int x, int y, int rx, int ry) = 0;
  virtual void GfxDrawRect(int x, int y, int w, int h) = 0;
  virtual void GfxDrawPolygon(const QPolygon & polygon) = 0;
  virtual void GfxFillCircle(int x, int y, int r) = 0;
  virtual void GfxFillEllipse(int x, int y, int rx, int ry) = 0;
  virtual void GfxFillRect(int x, int y, int w, int h) = 0;
  virtual void GfxFillPolygon(const QPolygon & polygon) = 0;
  virtual void GfxDrawText(int x, int y, const QString & text) = 0;
  virtual void GfxSetColor(int red, int green, int blue) = 0;
  virtual void GfxSetColor(const QString & name) = 0;
  virtual void GfxSetFillColor(int red, int green, int blue) = 0;
  virtual void GfxSetFillColor(const QString & name) = 0;
  virtual void GfxBeginUpdate() = 0;
  virtual void GfxEndUpdate() = 0;
  virtual void GfxSaveImg(const QString & fileName) = 0;

  virtual void DiaDiagramNew(const QString & diagramName) = 0;
  virtual void DiaDiagramDelete(const QString & diagramName) = 0;
  virtual void DiaDiagramSaveImg(const QString & diagramName, const QString & fileName) = 0;
  virtual void DiaDiagramSaveCsv(const QString & diagramName, const QString & fileName) = 0;
  virtual void DiaDiagramDeleteAll() = 0;
  virtual void DiaCurveNew(const QString & diagramName, const QString & curveName, int yAxis) = 0;
  virtual void DiaCurveDelete(const QString & diagramName, const QString & curveName) = 0;
  virtual void DiaCurveDeleteAll(const QString & diagramName) = 0;
  virtual void DiaCurveClearData(const QString & diagramName, const QString & curveName) = 0;
  virtual void DiaCurveSetData(const QString & diagramName, const QString &curveName, const QVector <double> & x, const QVector <double> & y) = 0;
  virtual void DiaCurveAppendData(const QString & diagramName, const QString &curveName, double x, double y) = 0;

  virtual void VecGfxClear() = 0;
  virtual void VecGfxSetColor(int red, int green, int blue) = 0;
  virtual void VecGfxSetColor(const QString & name) = 0;
  virtual void VecGfxSetFillColor(int red, int green, int blue) = 0;
  virtual void VecGfxSetFillColor(const QString & name) = 0;
  virtual void VecGfxDrawLine(qreal x1, qreal y1, qreal x2, qreal y2) = 0;
  virtual void VecGfxDrawCircle(qreal x, qreal y, qreal r) = 0;
  virtual void VecGfxDrawEllipse(qreal x, qreal y, qreal rx, qreal ry) = 0;
  virtual void VecGfxDrawRect(qreal x, qreal y, qreal w, qreal h) = 0;
  virtual void VecGfxDrawPolygon(const QPolygonF & polygon) = 0;
  virtual void VecGfxFillCircle(qreal x, qreal y, qreal r) = 0;
  virtual void VecGfxFillEllipse(qreal x, qreal y, qreal rx, qreal ry) = 0;
  virtual void VecGfxFillRect(qreal x, qreal y, qreal w, qreal h) = 0;
  virtual void VecGfxFillPolygon(const QPolygonF & polygon) = 0;
  virtual void VecGfxDrawText(qreal x, qreal y, const QString & text) = 0;
  virtual void VecGfxSaveImg(const QString & fileName, int width, int height) = 0;
  virtual void VecGfxSaveSvg(const QString & fileName) = 0;

  virtual bool ConsoleReadNumber(double *number) = 0;
  virtual bool ConsoleTestEof() = 0;
  virtual bool ConsoleReadLine(char *str, int maxNum) = 0;
  virtual int  ConsoleReadChars(char *str, int count) = 0;
};

//-----------------------------------------------------------------------------

#endif // INTERFACEGUI_H
