/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "gfxviewpixmap.h"

#include <iostream>
#include <QScrollBar>
#include <QMouseEvent>
#include <QWheelEvent>
#include <QLabel>
#include <QGraphicsPixmapItem>

using namespace std;

//-----------------------------------------------------------------------------

GfxItemPixmap::GfxItemPixmap(QGraphicsItem *parent) :
  QGraphicsItem(parent)
{
}

//-----------------------------------------------------------------------------

QRectF GfxItemPixmap::boundingRect() const
{
  return m_pixmap.rect();
}

//-----------------------------------------------------------------------------

void GfxItemPixmap::paint(QPainter *painter, const QStyleOptionGraphicsItem *item, QWidget *widget)
{
  (void)item;
  (void)widget;

  painter->drawPixmap(0, 0, m_pixmap);
}

//-----------------------------------------------------------------------------

GfxViewPixmap::GfxViewPixmap(QWidget *parent) :
  GfxView(parent),
  m_preventGfxUpdate(false),
  m_drawing(false),
  m_fill(false)
{
  m_gfxItemPixmap = new GfxItemPixmap();
  m_gfxScene.addItem(m_gfxItemPixmap);
  setScene(&m_gfxScene);

  SlotGfxSetSize(500, 500);
  SlotGfxClear();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSetSize(int x, int y)
{
  if (m_drawing)
  {
    m_painter.end();
    m_drawing = false;
    m_fill = false;
  }

  m_gfxItemPixmap->m_pixmap = QPixmap(x, y);
  m_gfxScene.update();

  setSceneRect(-x / 2, -y / 2, x * 2, y * 2);
  setTransform(QTransform());
  setTransformationAnchor(AnchorUnderMouse);
  centerOn(x / 2, y / 2);

  m_gfxScene.update();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxClear()
{
  BeginDraw();
  m_painter.eraseRect(m_gfxItemPixmap->m_pixmap.rect());
  EndDraw();

  centerOn(m_gfxItemPixmap->m_pixmap.width() / 2, m_gfxItemPixmap->m_pixmap.height() / 2);
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawPoint(int x, int y)
{
  BeginDraw();
  m_painter.drawPoint(x, y);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawLine(int x1, int y1, int x2, int y2)
{
  BeginDraw();
  m_painter.drawLine(x1, y1, x2, y2);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawCircle(int x, int y, int r)
{
  BeginDraw();
  m_painter.drawEllipse(x - r, y - r, r * 2, r * 2);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawEllipse(int x, int y, int rx, int ry)
{
  BeginDraw();
  m_painter.drawEllipse(x - rx, y - ry, rx * 2, ry * 2);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawRect(int x, int y, int w, int h)
{
  BeginDraw();
  m_painter.drawRect(x, y, w, h);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawPolygon(const QPolygon &polygon)
{
  BeginDraw();
  m_painter.drawPolygon(polygon);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxFillCircle(int x, int y, int r)
{
  BeginDraw(true);
  m_painter.drawEllipse(x - r, y - r, r * 2, r * 2);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxFillEllipse(int x, int y, int rx, int ry)
{
  BeginDraw(true);
  m_painter.drawEllipse(x - rx, y - ry, rx * 2, ry * 2);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxFillRect(int x, int y, int w, int h)
{
  BeginDraw(true);
  m_painter.drawRect(x, y, w, h);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxFillPolygon(const QPolygon &polygon)
{
  BeginDraw(true);
  m_painter.drawPolygon(polygon);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxDrawText(int x, int y, const QString &text)
{
  BeginDraw();
  m_painter.drawText(x, y, text);
  EndDraw();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSetColor(int red, int green, int blue)
{
  m_pen.setColor(QColor(red, green, blue));

  if (m_drawing)
    m_painter.setPen(m_pen);
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSetColor(const QString & name)
{
  m_pen.setColor(QColor(name));

  if (m_drawing)
    m_painter.setPen(m_pen);
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSetFillColor(int red, int green, int blue)
{
  m_brush.setColor(QColor(red, green, blue));

  if (m_drawing && m_fill)
    m_painter.setBrush(m_brush);
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSetFillColor(const QString & name)
{
  m_brush.setColor(QColor(name));

  if (m_drawing && m_fill)
    m_painter.setBrush(m_brush);
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxBeginUpdate()
{
  m_preventGfxUpdate = true;
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxEndUpdate()
{
  m_preventGfxUpdate = false;

  if (m_drawing)
  {
    m_painter.end();
    m_drawing = false;
    m_fill = false;
  }

  m_gfxScene.update();
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::BeginDraw(bool fill)
{
  if (!m_drawing)
  {
    m_painter.begin(&(m_gfxItemPixmap->m_pixmap));
    m_painter.setPen(m_pen);
    m_drawing = true;
    m_fill = false;
  }

  if (fill != m_fill)
  {
    m_fill = fill;

    if (fill)
      m_painter.setBrush(m_brush);
    else
      m_painter.setBrush(Qt::NoBrush);
  }
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::EndDraw()
{
  if (!m_preventGfxUpdate)
  {
    m_painter.end();
    m_drawing = false;
    m_fill = false;
    m_gfxScene.update();
  }
}

//-----------------------------------------------------------------------------

void GfxViewPixmap::SlotGfxSaveImg(const QString &fileName)
{
  m_gfxItemPixmap->m_pixmap.save(fileName);
}

//-----------------------------------------------------------------------------

