/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_func_tab.h"

#include "lauxlib.h"
#include "lstate.h"
#include "lua_helper.h"

#include "InterfaceGUI.h"

using namespace std;

//-----------------------------------------------------------------------------

int lua_func_tab_clear(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  iGui->TableClear();

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_tab_set_size(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  int   row = luaL_checkinteger(lua, 1);
  int   col = luaL_checkinteger(lua, 2);

  iGui->TableSetSize(row, col);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_tab_set_value(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  int   row = luaL_checkinteger(lua, 1);
  int   col = luaL_checkinteger(lua, 2);

  if (row < 1)
    lua_throw_error_from_c(lua, "row must be greater than 0");

  if (col < 1)
    lua_throw_error_from_c(lua, "column must be greater than 0");

  iGui->TableSetValue(row, col, luaL_checkstring(lua, 3));

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_tab_set_row_values(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;
  int   row = luaL_checkinteger(lua, 1);

  if (row < 1)
    lua_throw_error_from_c(lua, "row must be greater than 0");

  QList <QString> valueList;

  luaL_checktype(lua, 2, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
  {
    valueList.push_back(luaL_checkstring(lua, -1));
  }

  iGui->TableSetValuesInRow(row, valueList);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_tab_set_column_values(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;
  int   col = luaL_checkinteger(lua, 1);

  if (col < 1)
    lua_throw_error_from_c(lua, "column must be greater than 0");

  QList <QString> valueList;

  luaL_checktype(lua, 2, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
  {
    valueList.push_back(luaL_checkstring(lua, -1));
  }

  iGui->TableSetValuesInColumn(col, valueList);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_tab_save_csv(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       fileName = luaL_checkstring(lua, 1);

  iGui->TableSaveCsv(fileName);

  return 0;
}

//-----------------------------------------------------------------------------


