/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_helper.h"

#include "lauxlib.h"
#include "lualib.h"
#include "luasocket.h"

#include <stdio.h>
#include <ctype.h>
#include <iostream>

using namespace std;

//-----------------------------------------------------------------------------

lua_State * lua_init(void)
{
  lua_State *lua = lua_open();
  if (!lua)
  {
    cerr << "ERROR: can not initialize LUA!" << endl;
    return 0;
  }

  lua_gc(lua, LUA_GCSTOP, 0);

  luaL_openlibs(lua);
  
  luaopen_socket_core(lua);

  lua_gc(lua, LUA_GCRESTART, 0);

  return lua;
}

//-----------------------------------------------------------------------------

int lua_do_string(lua_State *lua, const char *str)
{
  int err = luaL_loadstring(lua, str) || lua_pcall(lua, 0, 0, 0);

  if (err != 0)
  {
    const char *msg = lua_tostring(lua, -1);
    if (msg)
      cerr << "ERROR: " << msg << endl;
    else
      cerr << "ERROR: unknown" << endl;

    lua_pop(lua, 1);
  }

  return err;
}

//-----------------------------------------------------------------------------

void lua_throw_error_from_c(lua_State *lua, const char *fmt, ...)
{
  static char err_msg[2048];
  va_list     ap;
  
  luaL_where(lua, 1);  
  
  va_start(ap, fmt);
  vsprintf(err_msg, fmt, ap);
  va_end(ap);
  
  lua_pushstring(lua, err_msg);
  
  lua_concat(lua, 2);
  lua_error(lua);
}

//-----------------------------------------------------------------------------
